'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.nodeFieldConfig = exports.TypeMapForRelayNode = undefined;
exports.composeWithRelay = composeWithRelay;

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql-compose/lib/graphql');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

var _wrapMutationResolver = require('./wrapMutationResolver');

var _wrapMutationResolver2 = _interopRequireDefault(_wrapMutationResolver);

var _globalId = require('./globalId');

var _nodeFieldConfig = require('./nodeFieldConfig');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// all wrapped typeComposers with Relay, stored in this variable
// for futher type resolving via NodeInterface.resolveType method

/* eslint-disable no-use-before-define */

var TypeMapForRelayNode = exports.TypeMapForRelayNode = {};
var nodeFieldConfig = exports.nodeFieldConfig = (0, _nodeFieldConfig.getNodeFieldConfig)(TypeMapForRelayNode);

function composeWithRelay(typeComposer) {
  if (!(typeComposer instanceof _graphqlCompose._ProtoTypeComposer)) {
    throw new Error('You should provide TypeComposer instance to composeWithRelay method');
  }

  if (typeComposer.getTypeName() === 'Query' || typeComposer.getTypeName() === 'RootQuery') {
    typeComposer.setField('node', nodeFieldConfig);
    return typeComposer;
  }

  if (typeComposer.getTypeName() === 'Mutation' || typeComposer.getTypeName() === 'RootMutation') {
    // just skip
    return typeComposer;
  }

  if (!typeComposer.hasRecordIdFn()) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) should have recordIdFn. ` + 'This function returns ID from provided object.');
  }

  var findById = typeComposer.getResolver('findById');
  if (!findById) {
    throw new Error(`TypeComposer(${typeComposer.getTypeName()}) provided to composeWithRelay ` + 'should have findById resolver.');
  }
  TypeMapForRelayNode[typeComposer.getTypeName()] = {
    resolver: findById,
    tc: typeComposer
  };

  typeComposer.addFields({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      description: 'The globally unique ID among all types',
      resolve: function resolve(source) {
        return (0, _globalId.toGlobalId)(typeComposer.getTypeName(), typeComposer.getRecordId(source));
      }
    }
  });

  typeComposer.addInterface(_nodeInterface2.default);

  typeComposer.getResolvers().forEach(function (resolver, resolverName) {
    if (resolver.kind === 'mutation') {
      var wrappedResolver = (0, _wrapMutationResolver2.default)(resolver, {
        resolverName,
        rootTypeName: typeComposer.getTypeName()
      });
      typeComposer.setResolver(resolverName, wrappedResolver);
    }
  });

  return typeComposer;
}