'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNodeFieldConfig = getNodeFieldConfig;

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql-compose/lib/graphql');

var _globalId = require('./globalId');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// this fieldConfig must be set to RootQuery.node field

/* eslint-disable no-param-reassign, import/prefer-default-export */
function getNodeFieldConfig(typeMapForRelayNode) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: _nodeInterface2.default,
    args: {
      id: {
        type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = (0, _globalId.fromGlobalId)(args.id),
          type = _fromGlobalId.type,
          id = _fromGlobalId.id;

      if (!typeMapForRelayNode[type]) {
        return null;
      }
      var _typeMapForRelayNode$ = typeMapForRelayNode[type],
          tc = _typeMapForRelayNode$.tc,
          findById = _typeMapForRelayNode$.resolver;

      if (findById && findById.resolve && tc) {
        var graphqlType = tc.getType();

        // set `returnType` to `info` for proper work of `getProjectionFromAST`
        // it will correctly add required fields for `relation` to `projection`
        var projection = void 0;
        if (info) {
          // $FlowFixMe
          info.returnType = graphqlType;
          projection = (0, _graphqlCompose.getProjectionFromAST)(info);
        } else {
          projection = {};
        }

        // suppose that first argument is argument with id field
        var idArgName = Object.keys(findById.args)[0];
        return findById.resolve({
          source,
          args: { [idArgName]: id }, // eg. mongoose has _id fieldname, so should map
          context,
          info,
          projection
        }).then(function (res) {
          if (!res) return res;
          res.__nodeType = graphqlType;
          return res;
        });
      }

      return null;
    }
  };
}