'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = wrapMutationResolver;

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql-compose/lib/graphql');

var _globalId = require('./globalId');

function upperFirst(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}
/* eslint-disable no-param-reassign */

function wrapMutationResolver(resolver, opts) {
  var resolverName = opts.resolverName,
      rootTypeName = opts.rootTypeName;


  function prepareArgs(newResolver) {
    var ITC = void 0;
    if (newResolver.args.input && newResolver.args.input.type) {
      var inputNamedType = (0, _graphql.getNamedType)(newResolver.args.input.type);
      if (inputNamedType instanceof _graphql.GraphQLInputObjectType) {
        ITC = new _graphqlCompose.InputTypeComposer(inputNamedType);
      }
    } else {
      // create input arg, and put into all current args

      ITC = _graphqlCompose.InputTypeComposer.create({
        name: `Relay${upperFirst(resolverName)}${rootTypeName}Input`,
        fields: newResolver.args
      });
      newResolver.setArgs({
        input: {
          name: 'input',
          // nonNull due required arg clientMutationId
          type: new _graphql.GraphQLNonNull(ITC.getType())
        }
      });
      // $FlowFixMe
      newResolver._relayIsArgsWrapped = true;
    }

    // add `clientMutationId` to args.input field
    if (ITC && !ITC.hasField('clientMutationId')) {
      ITC.setField('clientMutationId', {
        type: 'String',
        description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
      });
    }
  }

  function prepareResolve(newResolver, prevResolver) {
    newResolver.setResolve(function (resolveParams) {
      var clientMutationId = void 0;

      if (resolveParams && resolveParams.args) {
        if (resolveParams.args.input && resolveParams.args.input.clientMutationId) {
          clientMutationId = resolveParams.args.input.clientMutationId;
          delete resolveParams.args.input.clientMutationId;
        }
      }

      if (newResolver._relayIsArgsWrapped) {
        resolveParams.args = resolveParams.args.input;
      }

      return prevResolver.resolve(resolveParams).then(function (res) {
        res.nodeId = (0, _globalId.toGlobalId)(rootTypeName, res.recordId);
        if (clientMutationId) {
          res.clientMutationId = clientMutationId;
        }
        return res;
      });
    });
  }

  function prepareType(newResolver, prevResolver) {
    var outputType = prevResolver.getType();

    if (!(outputType instanceof _graphql.GraphQLObjectType)) {
      return;
    }

    var outputTC = new _graphqlCompose.TypeComposer(outputType);
    if (!outputTC.hasField('nodeId')) {
      outputTC.setField('nodeId', {
        type: 'ID',
        description: 'The globally unique ID among all types'
      });
    }

    if (!outputTC.hasField('clientMutationId')) {
      outputTC.setField('clientMutationId', {
        type: 'String',
        description: 'The client mutation ID used by clients like Relay to track the mutation. ' + 'If given, returned in the response payload of the mutation.'
      });
    }

    newResolver.setType(outputTC.getType());
  }

  return resolver.wrap(function (newResolver, prevResolver) {
    prepareArgs(newResolver);
    prepareResolve(newResolver, prevResolver);
    prepareType(newResolver, prevResolver);
    return newResolver;
  }, { name: 'RelayMutation' });
}